﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Collections;
using System.Windows.Threading;
using System.Windows;

namespace RealTimeDataChartSilverlight
{
    /// <summary>
    /// A dataset representing real-time data
    /// </summary>
    public class TickerBuffer : DependencyObject, INotifyCollectionChanged, IEnumerable
    {
        public int WindowSize
        {
            get
            {
                return _windowSize;
            }
            set
            {
                _windowSize = value;
            }
        }

        private IEnumerable _inputSource;
        private int _position = 0;
        private List<Row> _historicalDataSource = new List<Row>();
        private int _windowSize;

        /// <summary>
        /// Create a buffer using the data supplied, setting a buffer size of 500 items
        /// </summary>
        /// <param name="inputSource">A dataset to be used as input</param>
        public TickerBuffer(IEnumerable inputSource)
        {
            _inputSource = inputSource;
            _windowSize = 500;
        }

        /// <summary>
        /// Creates a buffer using the data supplied, and sets the buffer size to the specified value
        /// </summary>
        /// <param name="inputSource">A dataset to be used as input</param>
        /// <param name="windowSize">A value to be used for buffer size</param>
        public TickerBuffer(IEnumerable inputSource, int windowSize)
        {
            _inputSource = inputSource;
            _windowSize = windowSize;
        }

        /// <summary>
        /// Add a specified number of records containing randomly generated data to dataset
        /// </summary>
        /// <param name="recordCount"></param>
        public void Generate(int recordCount)
        {
            Random r = new Random();
            DateTime startDate = new DateTime(1990, 1, 1, 0, 0, 0);

            for (int i = 0; i < recordCount; ++i)
            {
                double baseValue = r.Next(100);
                double offsetValue = r.Next(5);
                _historicalDataSource.Add(new Row(startDate.AddSeconds(1).ToShortDateString(),baseValue, baseValue + offsetValue, baseValue - offsetValue, baseValue + 2*offsetValue, 0));
            }

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }

        /// <summary>
        /// Add a specified number of records from historical dataset to dataset
        /// </summary>
        /// <param name="recordCount"></param>
        public void AddRecordsFromHistoricalData(int recordCount)
        {
            //add records from historical data
            for (int i = 0; i < recordCount; ++i)
            {
                _historicalDataSource.Add((_inputSource as IList)[(_position) % (_inputSource as IList).Count] as Row);

                ++_position;
            }

            //remove old records from the beginning of the window according to window size
            while (_historicalDataSource.Count > _windowSize)
                _historicalDataSource.RemoveAt(0);

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }

        /// <summary>
        /// Add a specified number of records from a historical dataset to this dataset, starting from a specified position
        /// </summary>
        /// <param name="recordCount"></param>
        public void AddRecordsFromHistoricalData(int recordCount, int position)
        {
            _position = position;
            //add records from historical data
            for (int i = 0; i < recordCount; ++i)
            {
                _historicalDataSource.Add((_inputSource as IList)[(_position) % (_inputSource as IList).Count] as Row);

                ++_position;
            }

            //remove old records from the beginning of the window according to window size
            while (_historicalDataSource.Count > _windowSize)
                _historicalDataSource.RemoveAt(0);

            if (CollectionChanged != null)
            {
                CollectionChanged(this, new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
            }
        }



        #region IEnumerable Members

        public IEnumerator GetEnumerator()
        {
            return _historicalDataSource.GetEnumerator();
        }

        #endregion

        #region INotifyCollectionChanged Members

        public event NotifyCollectionChangedEventHandler CollectionChanged;

        #endregion
    }

    /// <summary>
    /// Row data schema
    /// </summary>
    public class Row : INotifyPropertyChanged
    {
        public Row(string date, double open, double high, double low, double close, double volume)
        {
            Column0 = DateTime.ParseExact(date, "M/d/yyyy", System.Globalization.CultureInfo.InvariantCulture);
            Column1 = open;
            Column2 = high;
            Column3 = low;
            Column4 = close;
            Column5 = volume;
        }

        public override string ToString()
        {
            return Column0.ToString() + ":" + Column1.ToString() + ":" + Column2.ToString() + ":" + Column3.ToString() + ":" + Column4.ToString() + ":" + Column5.ToString();
        }

        public DateTime Column0 { get { return column0; } set { column0 = value; RaisePropertyChanged("Column0"); } }
        private DateTime column0;

        public double Column1 { get { return column1; } set { column1 = value; RaisePropertyChanged("Column1"); } }
        private double column1;

        public double Column2 { get { return column2; } set { column2 = value; RaisePropertyChanged("Column2"); } }
        private double column2;

        public double Column3 { get { return column3; } set { column3 = value; RaisePropertyChanged("Column3"); } }
        private double column3;

        public double Column4 { get { return column4; } set { column4 = value; RaisePropertyChanged("Column4"); } }
        private double column4;

        public double Column5 { get { return column5; } set { column5 = value; RaisePropertyChanged("Column5"); } }
        private double column5;

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }
}
